/*
   Copyright 2018-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

*/

#include <irdb-core>
#include <irdb-util>
#include <map>
#include <set>
#include <memory>


namespace IRDB_SDK
{
	using namespace std;

	using AnalysisEngine_t = enum AnalysisEngine
		{
			aeSTARS,
			aeMoon,
			aeRida
		};

	using RegisterIDSet_t    = set<RegisterID_t>;
	using DeadRegisterMap_t  = map<Instruction_t*,RegisterIDSet_t>;
	using StaticGlobalStartMap_t  = map<Instruction_t*, VirtualOffset_t>;
	using RangeSentinelSet_t = InstructionSet_t;

	class DeepAnalysis_t
	{
		protected: 
			DeepAnalysis_t() {}
			DeepAnalysis_t(const DeepAnalysis_t& copy) = delete;

		public:
			virtual unique_ptr<FunctionSet_t          > getLeafFunctions()      const = 0;
			virtual unique_ptr<DeadRegisterMap_t      > getDeadRegisters()      const = 0;
			virtual unique_ptr<StaticGlobalStartMap_t > getStaticGlobalRanges() const = 0;
			virtual unique_ptr<RangeSentinelSet_t     > getRangeSentinels()     const = 0;

		// factories
			static unique_ptr<DeepAnalysis_t> factory(FileIR_t* firp, const AnalysisEngine_t& ae=aeSTARS, const vector<string>& options={});
	};
}
